module At_email ; end

### libs
require 'fileutils'
require "logger"
require 'digest'

### at_email libs
require "at_email/core"
require "at_email/version"
require "at_email/config"
require "at_email/account"
require "at_email/tasks"
require "at_email/threads"

module At_email

  DEFAULT_CONFIG_FILE = 'cfg/defaults/default_config.json'

end

module At_email

  class At_email::At_Logger

    attr_reader :log_file_default

    def initialize

      filename_timestamp = Time.now.strftime("%Y%m%d_%H%M%S")
      log_file_dir = $config[:output_base_dir] + '/_logs'

      @log_file_default = log_file_dir + '/' + $config[:task] + '.' + $config[:account_id] + '.' + filename_timestamp + '.log'
      if !Dir.exists?(log_file_dir)
        FileUtils.mkdir_p log_file_dir
      end
      @logger_stdout = Logger.new(STDOUT)
      @logger_stderr = Logger.new(STDERR)
      @logger_file_default = Logger.new(@log_file_default)
      @logger_stdout.datetime_format = '%Y-%m-%d %H:%M:%S'
      @logger_stdout.formatter = proc do |severity, datetime, progname, msg|
        "#{datetime} - " + $config[:task] + " - " + $config[:account_id] + " - #{msg}\n"
      end
      @logger_stderr.datetime_format = '%Y-%m-%d %H:%M:%S'
      @logger_stderr.formatter = proc do |severity, datetime, progname, msg|
        "#{datetime} - " + $config[:task] + " - " + $config[:account_id] + " - #{msg}\n"
      end
      @logger_file_default.datetime_format = '%Y-%m-%d %H:%M:%S'
      @logger_file_default.formatter = proc do |severity, datetime, progname, msg|
        "#{datetime} - " + $config[:task] + " - " + $config[:account_id] + " - #{msg}\n"
      end
      if $options.silent
        @logger_stdout.level = Logger::FATAL
        @logger_stderr.level = Logger::FATAL
        @logger_file_default.level = Logger::FATAL
      elsif $options.quiet
        @logger_stdout.level = Logger::WARN
        @logger_stderr.level = Logger::WARN
        @logger_file_default.level = Logger::WARN
      elsif $options.verbose
        @logger_stdout.level = Logger::DEBUG
        @logger_stderr.level = Logger::DEBUG
        @logger_file_default.level = Logger::DEBUG
      else
        @logger_stdout.level = Logger::INFO
        @logger_stderr.level = Logger::INFO
        @logger_file_default.level = Logger::INFO
      end
    end

    def debug(event)
      @logger_stdout.debug(event)
      @logger_file_default.debug(event)
    end

    def info(event)
      @logger_stdout.info(event)
      @logger_file_default.info(event)
    end

    def warn(event)
      @logger_stderr.warn(event)
      @logger_file_default.warn(event)
    end

    def error(event)
      @logger_stderr.error(event)
      @logger_file_default.error(event)
    end

    def fatal(event)
      @logger_stderr.fatal(event)
      @logger_file_default.fatal(event)
    end

    def unknown(event)
      @logger_stderr.unknown(event)
      @logger_file_default.unknown(event)
    end

    def event(event_level, event_tag, event_data)
      event_tag_formatted = get_event_tag_formatted(event_tag)
      if event_tag_formatted === ''
        log_string = event_data
      else
        log_string = event_tag_formatted + ' - ' + event_data
      end
      case event_level
      when 'DEBUG'
        $logger.debug log_string
      when 'WARN'
        $logger.error log_string
      when 'ERROR'
        $logger.fatal ''
        $logger.error log_string
        $logger.error ''
        $logger.error 'Error at line number: ' + __LINE__.to_s + ' of file: ' + __FILE__
        caller.each do |stack_line|
          $logger.error '     ' + stack_line
        end
        $logger.error ''
      when 'FATAL'
        error_code = 1
        $logger.fatal ''
        $logger.fatal log_string
        $logger.fatal ''
        $logger.fatal 'Fatal Error at line number: ' + __LINE__.to_s + ' of file: ' + __FILE__
        caller.each do |stack_line|
          $logger.fatal '     ' + stack_line
        end
        $logger.fatal ''
        $logger.fatal 'Exiting with error code: ' + error_code.to_s
        $logger.fatal ''
        puts "\n"
        exit(error_code)
      else
        $logger.info log_string
      end
    end

    def get_event_tag_formatted(event_tag)
      event_tag_formatted = ''
      if event_tag.length == 0
        event_tag_formatted = ''
      else
        if event_tag.length <= 2
          event_tag_formatted = '*' + event_tag + '*'
        else
          event_tag_formatted = '***** ' + event_tag + ' *****'
        end
      end
      return event_tag_formatted
    end

  end

  class At_email::RuntimeConfigure

    def configure_logger
      $logger = At_email::At_Logger.new()
    end

    def output_config
      $logger.debug ''
      $logger.debug 'Configuration'
      $config.each do |key, value|
        if key.to_s === 'password'
          $logger.debug ' ' + key.to_s + ' = ' + '*' * value.to_s.length
        else
          $logger.debug ' ' + key.to_s + ' = ' + value.to_s
        end
      end
      $logger.debug ''
    end

    def get_app_path
      $app_path = File.dirname(File.expand_path(File.dirname($0)))
    end

  end

  class At_email::Formatting

    def initialize
    end

    def get_hash(data, hash_length)
      if !hash_length.is_a? Numeric
        $logger.event('FATAL', 'FATAL ERROR', 'hash_length variable is not numeric')
        return false
      end
      if (hash_length > 32)
        $logger.event('FATAL', 'FATAL ERROR', 'hash_length maximum limit is 32 - Requested length: ' + hash_length.to_s)
        return false
      end
      return Digest::MD5.hexdigest(data)[0...hash_length]
    end

    def get_random_id(id_length)
      hash = get_hash(Time.now.to_f.to_s + '-' + rand(1..1000000000000).to_s, id_length)
      if hash
        return hash
      else
        return false
      end
    end

  end

end
